#include <sys/socket.h>
#include <sys/types.h>
#include <sys/event.h>
#include <sys/filio.h>

#include <sys/ioctl.h>
#include <netinet/in.h>
#include <stdio.h>
#include <string.h>
#include <err.h>

static void
wait_loop(int kq, int sock)
{
	struct kevent ev[32];
	struct sockaddr_in addr;
	socklen_t socklen;

	for (;;) {
		int nev = kevent(kq, NULL, 0, ev, 32, NULL);
		if (nev < 1)
			err(1, "kevent");
		for (int i = 0; i < nev; ++i) {
			if (ev[i].ident == sock) {
				printf("accept\n");
				int fd = accept(ev[i].ident,
				    (struct sockaddr *)&addr, &socklen);
				if (fd == -1)
					err(1, "accept");
			}
		}
	}
}

int
main()
{
	struct sockaddr_in addr;

	/* open a TCP socket */
	int kq = kqueue();

	int sock = socket(PF_INET, SOCK_STREAM, 0);

	struct kevent ev[2];
	EV_SET(&ev[0], sock, EVFILT_READ, EV_ADD | EV_CLEAR, 0, 0, NULL);
	EV_SET(&ev[1], sock, EVFILT_WRITE, EV_ADD | EV_CLEAR, 0, 0, NULL);

	int opt = 1;
	setsockopt(sock, SOL_SOCKET, SO_NOSIGPIPE, &opt, sizeof(opt));

	if (kevent(kq, ev, 2, NULL, 0, NULL) == -1)
	    err(1, "kevent");

	setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof(opt));

	memset(&addr, 0, sizeof(addr));
	addr.sin_port = htons(10000);

	bind(sock, (struct sockaddr *)&addr, sizeof(addr));
	listen(sock, 0x80);

	ioctl(sock, FIONBIO, &opt);

	if (kevent(kq, ev, 2, NULL, 0, NULL) == -1)
		err(1, "kevent");

	wait_loop(kq, sock);
}
